#ifndef _LOMAC_ATTR_H_
#define _LOMAC_ATTR_H_

/*************************************************************************
 *
 * lomac_attr.h
 *
 * LOMAC - Low Water-Mark Mandatory Access Control for Linux 
 * Copyright (c) 1999-2001 NAI Labs, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *   Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * 
 *   Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * 
 *   Neither the name of NAI Labs, Inc. nor the names of its contributors
 *   may be used to endorse or promote products derived from this software
 *   without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This is the public interface for the LOMAC attribute (lattr_t) ADT.
 *
 * LOMAC assigns attributes to all subjects and objects.  Architecture-
 * specific interfaces may store subject and object attributes in
 * different ways.  However, when the architecture-independent LOMAC
 * core manipulates attributes, it does so using this common format.
 *
 * Only subjects use the `trust' field.
 *
 *************************************************************************/

typedef struct {
  int level;        /* level (an integer range) */
  int categories;   /* category flags */
  int trust;        /* trust flags (meaningful for subjects only) */
} lattr_t;          /* lomac attribute structure type */


/* Constants for lattr_t's `level' field */
#define LOMAC_INVALID_LEVEL  0
#define LOMAC_LOWEST_LEVEL   1
#define LOMAC_HIGHEST_LEVEL  2

#define LEVEL_LEQ( a, b )( (a) <= (b) )

/* Constants for lattr_t's `categories' field */
#define LOMAC_CATEGORY_NONE  0x00000000
#define LOMAC_CATEGORY_ONE   0x00000010
#define LOMAC_CATEGORY_TWO   0x00000020
#define LOMAC_CATEGORY_THREE 0x00000040
#define LOMAC_CATEGORY_FOUR  0x00000080

/* Constants for lattr_t's `trust' field */
#define LOMAC_TRUST_NONE      0x00000000
#define LOMAC_TRUST_NODEMOTE  0x00000001

#define LATTR_HIGH    ((lattr_t){ LOMAC_HIGHEST_LEVEL, 0, 0 })
#define LATTR_LOW     ((lattr_t){ LOMAC_LOWEST_LEVEL,  0, 0 })
#define LATTR_INVALID ((lattr_t){ LOMAC_INVALID_LEVEL, 0, 0 })

#endif
