#ifndef _LOMAC_LOG_H_
#define _LOMAC_LOG_H_

/***************************************************************************
 *
 * lomac_log.h
 *
 * LOMAC - Low Water-Mark Mandatory Access Control for Linux 
 * Copyright (c) 1999-2001 NAI Labs, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *   Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * 
 *   Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * 
 *   Neither the name of NAI Labs, Inc. nor the names of its contributors
 *   may be used to endorse or promote products derived from this software
 *   without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *
 * This file contains the public interface to LOMAC's log functions
 *
 *
 ***************************************************************************/

#include "kernel_interface.h"

/* All log fxns share this common log buffer.  Some functions also *
 * use this buffer for scratch space.                              */
extern char log_s[];

/* Use this unsigned int and its constants to set the log output  *
 * verbosity.  Use of the log_* functions should be surrounded by *
 * if statements of the form "if( verbose & VERBOSE_FOO )" where  *
 * VERBOSE_FOO is the constant below which corresponds to the     *
 * type of event you're logging.                                  */
extern unsigned int verbose;
#define VERBOSE_NOLOG       0x00000000 /* no log output, please. */
#define VERBOSE_DEMOTE_DENY 0x00000001 /* log demotions and access denials. */
#define VERBOSE_PIPE        0x00000002 /* log changes to pipe "levels". */
#define VERBOSE_LOG_SOCKETS 0x00000004 /* log UNIX domain socket setup. */
#define VERBOSE_LOG_OBJECTS 0x00000008 /* log opening of objects. */
#ifdef TRUST
#define VERBOSE_TRUST       0x00000020 /* log when trust stops demotion. */
#endif

void log_start( void );
void log_append_string( const char *data_s );
void log_append_int( int data );
void log_append_subject_id( const lomac_subject_t *p_subject );
void log_append_object_id( const lomac_object_t *p_object );
void log_print( void );

#endif
