/**************************************************************************
 *
 * lomac_tpl.c
 *
 * LOMAC - Low Water-Mark Mandatory Access Control for Linux 
 * Copyright (c) 1999-2001 NAI Labs, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *   Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * 
 *   Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * 
 *   Neither the name of NAI Labs, Inc. nor the names of its contributors
 *   may be used to endorse or promote products derived from this software
 *   without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This is the implementation of the Trusted Program List (TPL).  It
 * provides a list of programs that are "trusted".
 *
 *
 * ## I've included XFree86 in the TPL.  This is a hack.  XFree86 needs
 * to create /var/log/XFree86.n.log while running at level 1.  /var/log
 * is high, so the creat operation fails.  Annoying.
 *
 **************************************************************************/

#ifdef TRUST

#include "policy_tpl.h"
#include "lomac_tpl.h"


/* is_running_trusted_program()
 *
 * in:     p_subject - subject to query
 * out:    nothing
 * return: value    condition
 *         -----    ---------
 *           1      `p_subject' is running a program in the TPL.
 *           0      `p_subject' is not running a program in the TPL, or error.
 *
 *    This function is a predicate indicating whether or not the
 * process specified by `p_subject' is running a program that is
 * listed in the tpl.  It is not implemented very efficiently.
 *
 */ 

int is_running_trusted_program( lomac_subject_t *p_subject ) {

  int index = 0;               /* index into the `tpl' array */
  lomac_object_t *p_binary;    /* `p_subject' is running this binary */
  lomac_object_t *p_tp_binary; /* points to binaries of trusted programs */
  int ret_val = 0;             /* value to be returned by this function */

#ifdef PARANOID
  if( !p_subject ) {
    PANIC( "LOMAC: is_running_trusted_program() given null subject\n" );
  }
#endif

  /* Get object corresponding to `p_subject's binary. */
  if( !( p_binary = get_subject_program_object( p_subject ) ) ) {
    return( 0 );    /* error, report not running trusted program */
  }

  index = 0;        /* start with first entry in TPL */
  ret_val = 0;      /* pessimistically assume not running trusted program */
  while( ( index < NUM_TPL_ENTRIES ) && ( ret_val == 0 ) ) {
    if( ( p_tp_binary = path_to_object( tpl[ index ] ) ) ) {
      if( p_tp_binary == p_binary ) {
	ret_val = 1;  /* yes, `p_subject' is running a trusted program */
      }
      object_done( p_tp_binary );
    }
    index++;        /* move to next TPL entry */
  }

  object_done( p_binary );
  return( ret_val ); 

} /* is_running_trusted_program() */

#endif /* TRUST */
